@@ -7,6 +7,7 @@ except:
 import logging
 import sys
 import os
+import collections
 # Hack to find pythoncom.dll - needed for some distribution/setups
 # E.g. if python is started with the full path outside of the python path, then it almost certainly fails
 cwd = os.getcwd()
@@ -286,7 +287,7 @@ class XLPython(object):
         exec(stmt, globals, locals)
 
 
-idle_queue = []
+idle_queue = collections.deque()
 idle_queue_event = win32event.CreateEvent(None, 0, 0, None)
 
 
@@ -324,45 +325,48 @@ def serve(clsid="{506e67c3-55b5-48c3-a035-eed5deea7d6d}"):
     msg = 'xlwings server running, clsid=%s'
     logger.info(msg, clsid) if logger.hasHandlers() else print(msg % clsid)
 
+    waitables = [idle_queue_event]
     while True:
+        timeout = TIMEOUT if idle_queue else win32event.INFINITE
         rc = win32event.MsgWaitForMultipleObjects(
-            [idle_queue_event],
+            waitables,
             0,
-            win32event.INFINITE,
+            timeout,
             win32event.QS_ALLEVENTS
         )
+        if rc == win32event.WAIT_OBJECT_0 or rc == win32event.WAIT_TIMEOUT:
+            while idle_queue:
+                task = idle_queue.popleft()
+                if not _execute_task(task):
+                    break
 
-        while True:
-            pythoncom.PumpWaitingMessages()
-
-            if not idle_queue:
-                break
-
-            task = idle_queue.pop(0)
-            _execute_task(task)
+        elif rc == win32event.WAIT_OBJECT_0 + len(waitables):
+            if pythoncom.PumpWaitingMessages():
+                break  # wm_quit
 
     pythoncom.CoRevokeClassObject(revokeId)
     pythoncom.CoUninitialize()
 
 
 def _execute_task(task):
+    """ Execute task. Returns False if task must be retried later. """
     try:
         task()
     except Exception as e:
-        if _ask_for_retry(e) and _can_retry(task):
+        if _ask_for_retry(e):
             msg = "Retrying TaskQueue '%s'."
             logger.info(msg, task) if logger.hasHandlers() else print(msg % task)
-            _execute_task(task)
+            idle_queue.appendleft(task)
+            return False
         else:
             import traceback
             msg = "TaskQueue '%s' threw an exception: %s"
             logger.error(msg, task, traceback.format_exc()) if logger.hasHandlers() else print(msg % (task, traceback.format_exc()))
+    return True
 
 
-def _can_retry(task):
-    return hasattr(task, 'nb_remaining_call') and task.nb_remaining_call > 0
-
 RPC_E_SERVERCALL_RETRYLATER = -2147418111
+TIMEOUT = 100  # ms
 
 
 def _ask_for_retry(exception):
